unit logs;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,linklists, ExtCtrls,math, StdCtrls;

const HistLength=100;

type

  TNetPoint=record
     up,down:integer;
     end;
  PNetPoint=^TNetPoint;
  TNetHist=class(TLinkList)
     procedure add(const up,down:integer);overload;
     procedure add(const point:TNetPoint);overload;
     function getfirst:TNetPoint;
     function getnext:TNetPoint;
     function getlast:TNetPoint;
     function getprev:TNetPoint;
     function getav:TNetPoint;
     end;

  TFLogs = class(TForm)
    Image1: TImage;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
    FFormStyle:TFormStyle;
    procedure setformstyle(const fs:TFormStyle);
  public
    { Public declarations }
    Hist:TNetHist;
    avhist:TNetHist;
    procedure AddPoint(const up,down:integer);
  published
    property FormStyle: TFormStyle read FFormStyle write SetFormStyle;
  end;

var
  FLogs: TFLogs;

implementation

uses main;

{$R *.dfm}

procedure TNetHist.add(const up,down:integer);
var P:PNetPoint;
begin
//add a new point to a history
new(p);
p.up:=up;
p.down:=down;
addend(p);
while length>histlength do begin
      getfirst;
      delete;
      end;
end;

procedure TNetHist.add(const point:TNetPoint);
var P:PNetPoint;
begin
//add a new point to a history
new(p);
p^:=point;
addend(p);
while length>histlength do begin
      getfirst;
      delete;
      end;
end;

function TNetHist.getfirst:TNetPoint;
begin
result:=PNetPoint(inherited getfirst)^;
end;

function TNetHist.getnext:TNetPoint;
begin
result:=PNetPoint(inherited getnext)^;
end;

function TNetHist.getprev:TNetPoint;
begin
result:=PNetPoint(inherited getprev)^;
end;

function TNetHist.getlast:TNetPoint;
begin
result:=PNetPoint(inherited getlast)^;
end;

function TNetHist.getav:TNetPoint;
var i:integer;
begin
//get the average of the last 4 points
if length<4 then result:=getlast
   else begin
   result.up:=getlast.up;
   result.down:=getlast.down;
   for i := 0 to 2 do
       with getprev do begin
            result.up:=result.up + up;
            result.down:=result.down + down;
            end;
   result.up:=result.up shr 2;
   result.down:=result.down shr 2;
   end;
end;

procedure TFLogs.AddPoint(const up,down:integer);
const gridint=15;
      widthmult=2;
var pt:TNetPoint;
    lx,luy,ldy,x,y:integer;
    maxud:integer;
    i:integer;
    lftborder:integer;
begin
//add a new point to the history and repaint the graph
//hist.add(up,down);
hist.add(up,down);
avhist.add(hist.getav);
if not flogs.Visible then exit;
maxud:=(max(u,d)*3 )shr 1;
lftborder:=image1.Picture.Bitmap.Canvas.TextWidth(format('%dK',[maxud]));
image1.Width:=histlength shl widthmult+lftborder;
image1.Height:=maxud;
image1.Picture.Bitmap.Width:=image1.Width;
image1.Picture.Bitmap.Height:=image1.Height;
with image1.Picture.Bitmap.Canvas do begin
     //clear image
     Brush.Color:=clblack;
     FillRect(rect(0,0,image1.Width,image1.Height));

     //draw background grid
     pen.Style:=psdot;
     pen.Color:=linecol;
     font.Color:=linecol;
     for i := 1 to maxud div gridint do begin
         y:=image1.Height-1-i*gridint;
         textout(0,y-textheight('1K') shr 1,format('%dK',[i*gridint]));
         moveto(lftborder,y);
         lineto(image1.Width,y);
         end;

     //draw upload limit
     pen.Color:=upcol;
     moveto(lftborder,image1.Height-1-u);
     lineto(image1.Width,image1.Height-1-u);
     //draw download limit
     pen.Color:=dncol;
     moveto(lftborder,image1.Height-1-d);
     lineto(image1.Width,image1.Height-1-d);

     //draw history
     pt:=hist.getfirst;
     lx:=image1.Width-(hist.Length-hist.CurrentNo) shl widthmult;
     ldy:=image1.height-1-pt.down;
     luy:=image1.height-1-pt.up;
     pen.Style:=pssolid;
     while pt.up<>-1 do begin
           x:=image1.Width-(hist.Length-hist.CurrentNo) shl widthmult;

           y:=image1.height-1-pt.up;
           pen.Color:=upcol2;
           moveto(lx,luy);
           lineto(x,y);
           luy:=y;

           y:=image1.height-1-pt.down;
           pen.Color:=dncol2;
           moveto(lx,ldy);
           lineto(x,y);
           ldy:=y;

           lx:=x;

           image1.Picture.Bitmap.Canvas.lineto(x,y);
           if hist.CurrentNo=hist.Length then pt.up:=-1
              else pt:=hist.getnext;
           end;

     //draw averaged history
     pt:=avhist.getfirst;
     lx:=image1.Width-(avhist.Length-avhist.CurrentNo) shl widthmult;
     ldy:=image1.height-1-pt.down;
     luy:=image1.height-1-pt.up;
     pen.Style:=pssolid;
     while pt.up<>-1 do begin
           x:=image1.Width-(avhist.Length-avhist.CurrentNo) shl widthmult;

           y:=image1.height-1-pt.up;
           pen.Color:=upcol;
           moveto(lx,luy);
           lineto(x,y);
           luy:=y;

           y:=image1.height-1-pt.down;
           pen.Color:=dncol;
           moveto(lx,ldy);
           lineto(x,y);
           ldy:=y;

           lx:=x;

           image1.Picture.Bitmap.Canvas.lineto(x,y);
           if avhist.CurrentNo=avhist.Length then pt.up:=-1
              else pt:=avhist.getnext;
           end;
     end;
image1.Invalidate;
end;

procedure TFLogs.FormCreate(Sender: TObject);
begin
hist:=tnethist.create;
avhist:=tnethist.create;
end;

procedure TFLogs.FormDestroy(Sender: TObject);
begin
hist.Free;
avhist.Free;
end;

procedure TFLogs.FormShow(Sender: TObject);
begin
if fmain.caot.Checked then flogs.FormStyle:=fsstayontop else flogs.FormStyle:=fsnormal;
timer1.Enabled:=true;
end;

procedure TFLogs.setformstyle(const fs:TFormStyle);
begin
fformstyle:=fs;
if not visible then exit;
case fs of
     fsNormal:SetWindowPos(Handle, HWND_NOTOPMOST, Left, Top, Width, Height,SWP_SHOWWINDOW);
     fsMDIChild, fsMDIForm:;
     fsStayOnTop:SetWindowPos(Handle, HWND_TOPMOST, Left, Top, Width, Height,SWP_SHOWWINDOW);
     end;
end;

procedure TFLogs.Timer1Timer(Sender: TObject);
begin
setformstyle(fformstyle);
end;

procedure TFLogs.FormHide(Sender: TObject);
begin
timer1.Enabled:=false;
end;

end.
